from typing import List

import docker

REQUIRED_CONTAINERS = [
    "elasticsearch-setup",
    "elasticsearch",
    "datahub-gms",
    "datahub-mce-consumer",
    "datahub-frontend-react",
    "datahub-mae-consumer",
    "kafka-topics-ui",
    "kafka-rest-proxy",
    "kafka-setup",
    "schema-registry-ui",
    "schema-registry",
    "broker",
    "kibana",
    "mysql",
    "neo4j",
    "zookeeper",
]

ALLOW_STOPPED = [
    "kafka-setup",
    "elasticsearch-setup",
]


def check_local_docker_containers() -> List[str]:
    issues: List[str] = []
    client = docker.from_env()

    containers = client.containers.list(
        all=True,
        filters={
            "label": "com.docker.compose.project=datahub",
        },
    )

    # Check number of containers.
    if len(containers) == 0:
        issues.append("quickstart.sh or dev.sh is not running")
    else:
        existing_containers = set(container.name for container in containers)
        missing_containers = set(REQUIRED_CONTAINERS) - existing_containers
        for missing in missing_containers:
            issues.append(f"{missing} container is not present")

    # Check that the containers are running and healthy.
    for container in containers:
        if container.name not in REQUIRED_CONTAINERS:
            # Ignores things like "datahub-frontend" which are no longer used.
            # This way, we only check required containers like "datahub-frontend-react"
            # even if there are some old containers lying around.
            continue
        if container.name in ALLOW_STOPPED:
            continue
        elif container.status != "running":
            issues.append(f"{container.name} is not running")
        elif "Health" in container.attrs["State"]:
            if container.attrs["State"]["Health"]["Status"] == "starting":
                issues.append(f"{container.name} is still starting")
            elif container.attrs["State"]["Health"]["Status"] != "healthy":
                issues.append(f"{container.name} is running but not healthy")

    return issues



from typing import List, Optional, Union

from airflow.exceptions import AirflowException
from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults

from datahub.integrations.airflow.hooks import DatahubKafkaHook, DatahubRestHook
from datahub.metadata.com.linkedin.pegasus2avro.mxe import MetadataChangeEvent


class DatahubBaseOperator(BaseOperator):
    ui_color = "#4398c8"

    hook: Union[DatahubRestHook, DatahubKafkaHook]

    @apply_defaults
    def __init__(
        self,
        *,
        datahub_rest_conn_id: Optional[str] = None,
        datahub_kafka_conn_id: Optional[str] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)

        if datahub_rest_conn_id and datahub_kafka_conn_id:
            raise AirflowException(
                "two hook conn id given when only exactly one required"
            )
        elif datahub_rest_conn_id:
            self.hook = DatahubRestHook(datahub_rest_conn_id)
        elif datahub_kafka_conn_id:
            self.hook = DatahubKafkaHook(datahub_kafka_conn_id)
        else:
            raise AirflowException("no hook conn id provided")


class DatahubEmitterOperator(DatahubBaseOperator):
    @apply_defaults
    def __init__(
        self,
        mces: List[MetadataChangeEvent],
        datahub_rest_conn_id: Optional[str] = None,
        datahub_kafka_conn_id: Optional[str] = None,
        **kwargs,
    ):
        super().__init__(
            datahub_rest_conn_id=datahub_rest_conn_id,
            datahub_kafka_conn_id=datahub_kafka_conn_id,
            **kwargs,
        )
        self.mces = mces

    def execute(self, context):
        self.hook.emit_mces(self.mces)



import datahub


def get_provider_info():
    return {
        "name": "DataHub",
        "description": "`DataHub <https://datahubproject.io/>`__\n",
        "hook-class-names": [
            "datahub.integrations.airflow.hooks.DatahubRestHook",
            "datahub.integrations.airflow.hooks.DatahubKafkaHook",
        ],
        "package-name": datahub.__package_name__,
        "versions": [datahub.__version__],
    }



from typing import IO

import yaml

from datahub.configuration import ConfigurationMechanism


class YamlConfigurationMechanism(ConfigurationMechanism):
    """Ability to load configuration from yaml files"""

    def load_config(self, config_fp: IO):
        config = yaml.safe_load(config_fp)
        return config



import sys

import click

from datahub.check.docker import check_local_docker_containers
from datahub.check.json_file import check_mce_file


@click.group()
def check() -> None:
    pass


@check.command()
@click.argument("json-file", type=click.Path(exists=True, dir_okay=False))
def mce_file(json_file: str) -> None:
    """Check the schema of a MCE JSON file"""

    report = check_mce_file(json_file)
    click.echo(report)


@check.command()
def local_docker() -> None:
    """Check that the local Docker containers are healthy"""

    issues = check_local_docker_containers()
    if not issues:
        click.secho("✔ No issues detected", fg="green")
    else:
        click.secho("The following issues were detected:", fg="bright_red")
        for issue in issues:
            click.echo(f"- {issue}")
        sys.exit(1)
